@@ -36,11 +36,36 @@ A few obvious properties that are not currently handled realistically:
 - Symlinks and binary files are ignored
 '''
 
-import bisect, collections, itertools, json, os, random, time, sys
-from mercurial import cmdutil, context, patch, scmutil, util, hg
-from mercurial.i18n import _
-from mercurial.node import nullrev, nullid, short
+from __future__ import absolute_import
+import bisect
+import collections
+import itertools
+import json
+import os
+import random
+import sys
+import time
 
+from mercurial.i18n import _
+from mercurial.node import (
+    nullid,
+    nullrev,
+    short,
+)
+from mercurial import (
+    cmdutil,
+    context,
+    error,
+    hg,
+    patch,
+    scmutil,
+    util,
+)
+
+# Note for extension authors: ONLY specify testedwith = 'internal' for
+# extensions which SHIP WITH MERCURIAL. Non-mainline extensions should
+# be specifying the version(s) of Mercurial they are tested with, or
+# leave the attribute unspecified.
 testedwith = 'internal'
 
 cmdtable = {}
@@ -249,8 +274,8 @@ def synthesize(ui, repo, descpath, **opts):
     '''
     try:
         fp = hg.openpath(ui, descpath)
-    except Exception, err:
-        raise util.Abort('%s: %s' % (descpath, err[0].strerror))
+    except Exception as err:
+        raise error.Abort('%s: %s' % (descpath, err[0].strerror))
     desc = json.load(fp)
     fp.close()
 
@@ -281,8 +306,8 @@ def synthesize(ui, repo, descpath, **opts):
     dictfile = opts.get('dict') or '/usr/share/dict/words'
     try:
         fp = open(dictfile, 'rU')
-    except IOError, err:
-        raise util.Abort('%s: %s' % (dictfile, err.strerror))
+    except IOError as err:
+        raise error.Abort('%s: %s' % (dictfile, err.strerror))
     words = fp.read().splitlines()
     fp.close()
 
@@ -359,7 +384,10 @@ def synthesize(ui, repo, descpath, **opts):
                             files.iterkeys(), filectxfn, ui.username(),
                             '%d %d' % util.makedate())
         initnode = mc.commit()
-        hexfn = ui.debugflag and hex or short
+        if ui.debugflag:
+            hexfn = hex
+        else:
+            hexfn = short
         ui.status(_('added commit %s with %d files\n')
                   % (hexfn(initnode), len(files)))
 
@@ -475,8 +503,11 @@ def renamedirs(dirs, words):
         if dirpath in replacements:
             return replacements[dirpath]
         head, _ = os.path.split(dirpath)
-        head = head and rename(head) or ''
-        renamed = os.path.join(head, wordgen.next())
+        if head:
+            head = rename(head)
+        else:
+            head = ''
+        renamed = os.path.join(head, next(wordgen))
         replacements[dirpath] = renamed
         return renamed
     result = []
