@@ -277,10 +277,22 @@ All the above add a comment to the Bugzilla bug record of the form::
     Changeset commit comment. Bug 1234.
 '''
 
+from __future__ import absolute_import
+
+import re
+import time
+
 from mercurial.i18n import _
 from mercurial.node import short
-from mercurial import cmdutil, mail, util, error
-import re, time, urlparse, xmlrpclib
+from mercurial import (
+    cmdutil,
+    error,
+    mail,
+    util,
+)
+
+urlparse = util.urlparse
+xmlrpclib = util.xmlrpclib
 
 # Note for extension authors: ONLY specify testedwith = 'internal' for
 # extensions which SHIP WITH MERCURIAL. Non-mainline extensions should
@@ -875,8 +887,10 @@ class bugzilla(object):
                 count -= 1
             return root
 
-        mapfile = self.ui.config('bugzilla', 'style')
+        mapfile = None
         tmpl = self.ui.config('bugzilla', 'template')
+        if not tmpl:
+            mapfile = self.ui.config('bugzilla', 'style')
         if not mapfile and not tmpl:
             tmpl = _('changeset {node|short} in repo {root} refers '
                      'to bug {bug}.\ndetails:\n\t{desc|tabindent}')
