@@ -9,6 +9,7 @@
 from __future__ import absolute_import
 
 import errno
+import hashlib
 
 from .i18n import _
 from .node import short
@@ -17,14 +18,13 @@ from . import (
     changegroup,
     error,
     exchange,
+    obsolete,
     util,
 )
 
 def _bundle(repo, bases, heads, node, suffix, compress=True):
     """create a bundle with the specified revisions as a backup"""
-    cgversion = '01'
-    if 'generaldelta' in repo.requirements:
-        cgversion = '02'
+    cgversion = changegroup.safeversion(repo)
 
     cg = changegroup.changegroupsubset(repo, bases, heads, 'strip',
                                        version=cgversion)
@@ -36,7 +36,7 @@ def _bundle(repo, bases, heads, node, suffix, compress=True):
     # Include a hash of all the nodes in the filename for uniqueness
     allcommits = repo.set('%ln::%ln', bases, heads)
     allhashes = sorted(c.hex() for c in allcommits)
-    totalhash = util.sha1(''.join(allhashes)).hexdigest()
+    totalhash = hashlib.sha1(''.join(allhashes)).hexdigest()
     name = "%s/%s-%s-%s.hg" % (backupdir, short(node), totalhash[:8], suffix)
 
     comp = None
@@ -48,7 +48,7 @@ def _bundle(repo, bases, heads, node, suffix, compress=True):
         bundletype = "HG10BZ"
     else:
         bundletype = "HG10UN"
-    return changegroup.writebundle(repo.ui, cg, name, bundletype, vfs,
+    return bundle2.writebundle(repo.ui, cg, name, bundletype, vfs,
                                    compression=comp)
 
 def _collectfiles(repo, striprev):
@@ -74,7 +74,8 @@ def _collectbrokencsets(repo, files, striprev):
     return s
 
 def strip(ui, repo, nodelist, backup=True, topic='backup'):
-
+    # This function operates within a transaction of its own, but does
+    # not take any lock on the repo.
     # Simple way to maintain backwards compatibility for this
     # argument.
     if backup in ['none', 'strip']:
@@ -159,26 +160,29 @@ def strip(ui, repo, nodelist, backup=True, topic='backup'):
         msg = _('programming error: cannot strip from inside a transaction')
         raise error.Abort(msg, hint=_('contact your extension maintainer'))
 
-    tr = repo.transaction("strip")
-    offset = len(tr.entries)
-
     try:
-        tr.startgroup()
-        cl.strip(striprev, tr)
-        mfst.strip(striprev, tr)
-        for fn in files:
-            repo.file(fn).strip(striprev, tr)
-        tr.endgroup()
+        with repo.transaction("strip") as tr:
+            offset = len(tr.entries)
+
+            tr.startgroup()
+            cl.strip(striprev, tr)
+            mfst.strip(striprev, tr)
+            if 'treemanifest' in repo.requirements: # safe but unnecessary
+                                                    # otherwise
+                for unencoded, encoded, size in repo.store.datafiles():
+                    if (unencoded.startswith('meta/') and
+                        unencoded.endswith('00manifest.i')):
+                        dir = unencoded[5:-12]
+                        repo.dirlog(dir).strip(striprev, tr)
+            for fn in files:
+                repo.file(fn).strip(striprev, tr)
+            tr.endgroup()
 
-        try:
             for i in xrange(offset, len(tr.entries)):
                 file, troffset, ignore = tr.entries[i]
                 repo.svfs(file, 'a').truncate(troffset)
                 if troffset == 0:
                     repo.store.markremoved(file)
-            tr.close()
-        finally:
-            tr.release()
 
         if saveheads or savebases:
             ui.note(_("adding branch\n"))
@@ -188,20 +192,29 @@ def strip(ui, repo, nodelist, backup=True, topic='backup'):
                 # silence internal shuffling chatter
                 repo.ui.pushbuffer()
             if isinstance(gen, bundle2.unbundle20):
-                tr = repo.transaction('strip')
-                tr.hookargs = {'source': 'strip',
-                               'url': 'bundle:' + vfs.join(chgrpfile)}
-                try:
+                with repo.transaction('strip') as tr:
+                    tr.hookargs = {'source': 'strip',
+                                   'url': 'bundle:' + vfs.join(chgrpfile)}
                     bundle2.applybundle(repo, gen, tr, source='strip',
                                         url='bundle:' + vfs.join(chgrpfile))
-                    tr.close()
-                finally:
-                    tr.release()
             else:
                 gen.apply(repo, 'strip', 'bundle:' + vfs.join(chgrpfile), True)
             if not repo.ui.verbose:
                 repo.ui.popbuffer()
             f.close()
+        repo._phasecache.invalidate()
+
+        for m in updatebm:
+            bm[m] = repo[newbmtarget].node()
+        lock = tr = None
+        try:
+            lock = repo.lock()
+            tr = repo.transaction('repair')
+            bm.recordchange(tr)
+            tr.close()
+        finally:
+            tr.release()
+            lock.release()
 
         # remove undo files
         for undovfs, undofile in repo.undofiles():
@@ -212,9 +225,6 @@ def strip(ui, repo, nodelist, backup=True, topic='backup'):
                     ui.warn(_('error removing %s: %s\n') %
                             (undovfs.join(undofile), str(e)))
 
-        for m in updatebm:
-            bm[m] = repo[newbmtarget].node()
-        bm.write()
     except: # re-raises
         if backupfile:
             ui.warn(_("strip failed, full bundle stored in '%s'\n")
@@ -242,8 +252,7 @@ def rebuildfncache(ui, repo):
                   'support fncache)\n'))
         return
 
-    lock = repo.lock()
-    try:
+    with repo.lock():
         fnc = repo.store.fncache
         # Trigger load of fncache.
         if 'irrelevant' in fnc:
@@ -255,7 +264,8 @@ def rebuildfncache(ui, repo):
 
         repolen = len(repo)
         for rev in repo:
-            ui.progress(_('changeset'), rev, total=repolen)
+            ui.progress(_('rebuilding'), rev, total=repolen,
+                        unit=_('changesets'))
 
             ctx = repo[rev]
             for f in ctx.files():
@@ -272,7 +282,17 @@ def rebuildfncache(ui, repo):
                 if repo.store._exists(d):
                     newentries.add(d)
 
-        ui.progress(_('changeset'), None)
+        ui.progress(_('rebuilding'), None)
+
+        if 'treemanifest' in repo.requirements: # safe but unnecessary otherwise
+            for dir in util.dirs(seenfiles):
+                i = 'meta/%s/00manifest.i' % dir
+                d = 'meta/%s/00manifest.d' % dir
+
+                if repo.store._exists(i):
+                    newentries.add(i)
+                if repo.store._exists(d):
+                    newentries.add(d)
 
         addcount = len(newentries - oldentries)
         removecount = len(oldentries - newentries)
@@ -287,16 +307,10 @@ def rebuildfncache(ui, repo):
             fnc.entries = newentries
             fnc._dirty = True
 
-            tr = repo.transaction('fncache')
-            try:
+            with repo.transaction('fncache') as tr:
                 fnc.write(tr)
-                tr.close()
-            finally:
-                tr.release()
         else:
             ui.write(_('fncache already up to date\n'))
-    finally:
-        lock.release()
 
 def stripbmrevset(repo, mark):
     """
@@ -309,3 +323,32 @@ def stripbmrevset(repo, mark):
                      "ancestors(head() and not bookmark(%s)) - "
                      "ancestors(bookmark() and not bookmark(%s))",
                      mark, mark, mark)
+
+def deleteobsmarkers(obsstore, indices):
+    """Delete some obsmarkers from obsstore and return how many were deleted
+
+    'indices' is a list of ints which are the indices
+    of the markers to be deleted.
+
+    Every invocation of this function completely rewrites the obsstore file,
+    skipping the markers we want to be removed. The new temporary file is
+    created, remaining markers are written there and on .close() this file
+    gets atomically renamed to obsstore, thus guaranteeing consistency."""
+    if not indices:
+        # we don't want to rewrite the obsstore with the same content
+        return
+
+    left = []
+    current = obsstore._all
+    n = 0
+    for i, m in enumerate(current):
+        if i in indices:
+            n += 1
+            continue
+        left.append(m)
+
+    newobsstorefile = obsstore.svfs('obsstore', 'w', atomictemp=True)
+    for bytes in obsolete.encodemarkers(left, True, obsstore._version):
+        newobsstorefile.write(bytes)
+    newobsstorefile.close()
+    return n
