@@ -1,19 +1,27 @@
 #!/usr/bin/env python
 """
-Tests the behaviour of filelog w.r.t. data starting with '\1\n'
+Tests the behavior of filelog w.r.t. data starting with '\1\n'
 """
-from mercurial import ui, hg
-from mercurial.node import nullid, hex
+from __future__ import absolute_import, print_function
 
-myui = ui.ui()
+from mercurial.node import (
+    hex,
+    nullid,
+)
+from mercurial import (
+    hg,
+    ui as uimod,
+)
+
+myui = uimod.ui()
 repo = hg.repository(myui, path='.', create=True)
 
 fl = repo.file('foobar')
 
 def addrev(text, renamed=False):
     if renamed:
-        # data doesnt matter. Just make sure filelog.renamed() returns True
-        meta = dict(copyrev=hex(nullid), copy='bar')
+        # data doesn't matter. Just make sure filelog.renamed() returns True
+        meta = {'copyrev': hex(nullid), 'copy': 'bar'}
     else:
         meta = {}
 
@@ -30,7 +38,7 @@ def addrev(text, renamed=False):
             lock.release()
 
 def error(text):
-    print 'ERROR: ' + text
+    print('ERROR: ' + text)
 
 textwith = '\1\nfoo'
 without = 'foo'
@@ -52,4 +60,4 @@ if fl.cmp(node, textwith) or not fl.cmp(node, without):
 if fl.size(1) != len(textwith):
     error('filelog.size for a renaming + data starting with \\1\\n')
 
-print 'OK.'
+print('OK.')
